<?php
// Sanity check - check for MediaWiki environment...
if( !defined( 'MEDIAWIKI' ) )
die( "This is an extension to the MediaWiki package and cannot be run standalone." );


class Upload3D extends SpecialUpload {
	/**
	 * Constructor
	 */
	public function __construct() {
		global $wgRequest, $wgMimeTypeBlacklist, $wgFileExtensions;
		// Adjust some parameters
		error_reporting(E_ERROR | E_PARSE);
		$wgFileExtensions= array('zip');
		foreach($wgMimeTypeBlacklist as $name=>$value){
			if($value=='application/zip'){
				unset($wgMimeTypeBlacklist[$name]);
			}
		}

		parent::SpecialPage( 'Upload3D'/*class*/, 'upload'/*restriction*/, true, false, dirname(__FILE__).'/SpecialUpload3D.body.php');

		parent::loadRequest( is_null( $request ) ? $wgRequest : $request );
	}

	/**
	 * Get an UploadForm instance with title and text properly set.
	 *
	 * @param string $message HTML string to add to the form
	 * @param string $sessionKey Session key in case this is a stashed upload
	 * @return UploadForm
	 */
	protected function getUploadForm( $message = '', $sessionKey = '', $hideIgnoreWarning = false ) {
		global $wgOut;

		# Initialize form
		$form = new Upload3DForm( array(
			'watch' => $this->getWatchCheck(), 
			'forreupload' => $this->mForReUpload, 
			'sessionkey' => $sessionKey,
			'hideignorewarning' => $hideIgnoreWarning,
			'destwarningack' => (bool)$this->mDestWarningAck,
			
			'texttop' => $this->uploadFormTextTop,
			'textaftersummary' => $this->uploadFormTextAfterSummary,
			'destfile' => $this->mDesiredDestName,
		) );
		$form->setTitle( $this->getTitle() );

		# Check the token, but only if necessary
		if( !$this->mTokenOk && !$this->mCancelUpload
		&& ( $this->mUpload && $this->mUploadClicked ) ) {
			$form->addPreText( wfMsgExt( 'session_fail_preview', 'parseinline' ) );
		}

		# Add text to form
		$form->addPreText( '<div id="uploadtext">' .
		wfMsgExt( 'uploadtext', 'parse', array( $this->mDesiredDestName ) ) .
			'</div>' );
		# Add upload error message
		$form->addPreText( $message );

		# Add footer to form
		$uploadFooter = wfMsgNoTrans( 'uploadfooter' );
		if ( $uploadFooter != '-' && !wfEmptyMsg( 'uploadfooter', $uploadFooter ) ) {
			$form->addPostText( '<div id="mw-upload-footer-message">'
			. $wgOut->parse( $uploadFooter ) . "</div>\n" );
		}

		return $form;

	}

	/**
	 * Really do the upload
	 * Checks are made in SpecialUpload::execute()
	 */
	function processUpload() {
		global $wgOut, $wgRequest, $_FILES, $wgFileExtensions;
		$wgFileExtensions[]='png';
		$wgFileExtensions[]='stl';
		$this->mIgnoreWarning=true;
		$filename=$_FILES['wpUploadFile']['tmp_name'];
		$dir = dirname(__FILE__);
		//Try to open the zip
		$zip = new ZipArchive;
		$res = $zip->open($filename);
		if ($res === TRUE) {
			// extract the files
			$zip->extractTo($dir.'/unzip/');
			$zip->close();
			// open this directory
			$myDirectory = opendir($dir."/unzip");
			// get filenames
			$wgOut->addHTML('<table>');
			$fileNames=array();
			while($entryName = readdir($myDirectory)) {
				if ($entryName != "." && $entryName != "..")
				$fileNames[] = $entryName;
			}
			closedir($myDirectory);
			//loop over all the files
			foreach($fileNames as $file){
				//check if it is an stl file
				if(strstr($file, '.stl')){
					unset($picture);
					$picName=str_replace(".stl", ".png", $file);
					//search for the picture to go with it
					foreach($fileNames as $pic){
						if($pic==$picName){
							$picture=$pic;
							$wgOut->addHTML("Matching image found for $file: $pic.<br />\n");
						}
					}
					//there was no picture, so it needs to be created
					if(!isset($picture)){
						$wgOut->addHTML("Converting $file to $picName.<br />\n");
						$this->convert($file, $picName, "$dir/unzip/");
					}

					//let mediawiki know the files have been uploaded
					$this->processFile($file, "$dir/unzip", 'application/vnd.ms-pki.stl');
					$this->processFile($picName, "$dir/unzip", 'image/png');
				}
			}

			//Delete all the remaining files
			foreach($fileNames as $file){
				if(file_exists("$dir/unzip/$file"))
				unlink("$dir/unzip/$file");
				if(is_dir("$dir/unzip/$file"))
				$this->delete_directory("$dir/unzip/$file");
			}
			$wgOut->addHTML('</table>');
			if( $this->mHasWarning ) {
				$this->showWarningOptions();
			}
			$wgOut->addHTML('<br />Your files have been uploaded. You can find them in the <a href="/wiki/index.php/Special:ListFiles" title="Special:ListFiles">list of uploaded files</a>.');
		} else {
			$wgOut->addHTML("Unable to open zip file.");
		}
	}

	/**
	 * This function deletes a directory
	 *
	 * @param $dirname	name of the directory
	 */
	function delete_directory($dirname) {
		if (is_dir($dirname))
		$dir_handle = opendir($dirname);
		if (!$dir_handle)
		return false;
		while($file = readdir($dir_handle)) {
			if ($file != "." && $file != "..") {
				if (!is_dir($dirname."/".$file))
				unlink($dirname."/".$file);
				else
				$this->delete_directory($dirname.'/'.$file);
			}
		}
		closedir($dir_handle);
		rmdir($dirname);
		return true;
	}

	/**
	 * This function is a workaround to embed the multiple files in the wiki. 
	 * Feel free to replace it with cleaner code instead of this hack.
	 *
	 * @param $name		name of the file
	 * @param $dir		directory of the file
	 * @param $filetype	filetype of the file you are uploading
	 */
	function processFile($name, $dir, $filetype){
		global $wgOut, $_FILES, $_POST;
		$_FILES['wpUploadFile']['name']		=$name;
		$_FILES['wpUploadFile']['type']		=$filetype;
		$_FILES['wpUploadFile']['tmp_name']	="$dir/$name";
		$_FILES['wpUploadFile']['error']	=0;
		$_FILES['wpUploadFile']['size']		=filesize("$dir/$name");
		$_POST['wpDestFile']				=$name;

		$this->mRequest 		= new WebRequest();
		$this->loadRequest($this->mRequest);
		$this->mLocalFile = $this->mUpload->getLocalFile();

		$this->mTempPath 		= "$dir/$name";
		$this->mFileSize 		= filesize("$dir/$name");
		$this->mSrcName 		= $name; // for mw > 1.9
		$this->mRemoveTempFile 	= true;
		$this->mIgnoreWarning 	= true;
		$this->mUploadError 	= false;
		$this->mDesiredDestName = $name;
		$this->mFileProps		= false;

		$wgOut->addHTML( '<tr><td>' );
		$status = $this->mLocalFile->upload( $this->mTempPath, $comment, $pageText, File::DELETE_SOURCE, $this->mFileProps, false, $user );

		if( $status->isGood() && $watch ) {
			$user->addWatch( $this->getLocalFile()->getTitle() );
		}

		if( $status->isGood() ) {
			wfRunHooks( 'UploadComplete', array( &$this ) );
		}

		if ( !$status->isGood() ) {
			$this->showUploadError( $wgOut->parse( $status->getWikiText() ) );
			return;
		}

		$wgOut->addHTML( '</td></tr>' );
	}

	/**
	 * Converts an stl file to a png file
	 *
	 * @param $filename		the stl file
	 * @param $output		the output location
	 * @param $dir			the directory where everything happens
	 */
	function convert($filename, $output, $dir){
		//Convert the stl to an includable file for povray
		$stl2pov= "stl2pov -s {$dir}{$filename} > {$dir}render.inc";
		exec($stl2pov);

		//Rename the model in the newly created .inc file to MODELNAME
		$contents = file_get_contents("{$dir}render.inc");
		$startReplacing = strpos($contents, '#declare');
		$stopReplacing = strpos($contents, '{', $startReplacing);
		$toReplace=substr($contents, $startReplacing, $stopReplacing-$startReplacing);
		$contents = str_replace($toReplace, '#declare MODELNAME = mesh ', $contents);
		$handle=fopen("{$dir}render.inc", 'w');
		fwrite($handle, $contents);
		fclose($handle);

		//Render the file
		$basedir= substr($dir, 0, strlen($dir)-1);
		$basedir= substr($basedir, 0, strrpos($basedir, '/')+1);
		$povray = "povray -i\"{$basedir}render.pov\" +W600 +H600 -o\"{$dir}{$output}\" +GA -D";
		exec($povray);

		//Delete the include-file
		unlink("{$dir}render.inc");
	}
}

/**
 * Shows the upload form for this extension. You could overwrite more functions to give a better form.
 * 
 * @ingroup SpecialPage
 */
class Upload3DForm extends UploadForm {
	function displayForm( $submitResult ) {
		global $wgOut;
		$this->mPre = '<div id="uploadtext">
<p>Use the form below to upload a zip with stl and png files in it. It is not required to add png images. If they are present, they will be used. Otherwise they will be rendered. An image that is a render of a given stl file should have exactly the same name, this is case sensitive. If the names don\'t match, the image will be ignored and a new one will be rendered.</p>
<p>Although an option is shown to ignore warnings, you should always make the script ignore warnings by selecting the box. Otherwise it won\'t work. This means your file will overwrite existing files with the same name.</p>
<p>To view or search previously uploaded files go to the <a href="/wiki/index.php/Special:ListFiles" title="Special:ListFiles">list of uploaded files</a>, (re)uploads are also logged in the <a href="/wiki/index.php/Special:Log/upload" title="Special:Log/upload">upload log</a>, deletions in the <a href="/wiki/index.php/Special:Log/delete" title="Special:Log/delete">deletion log</a>.
</div>';
		parent::displayForm();
	}
}